'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var path = require('path');
var fs = require('fs-extra');
var pluginScaffolderBackend = require('@backstage/plugin-scaffolder-backend');
var pluginScaffolderNode = require('@backstage/plugin-scaffolder-node');
var backendCommon = require('@backstage/backend-common');
var yaml = require('js-yaml');
var zod = require('zod');
var https = require('https');
var url = require('url');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

function _interopNamespace(e) {
	if (e && e.__esModule) return e;
	var n = Object.create(null);
	if (e) {
		Object.keys(e).forEach(function (k) {
			if (k !== 'default') {
				var d = Object.getOwnPropertyDescriptor(e, k);
				Object.defineProperty(n, k, d.get ? d : {
					enumerable: true,
					get: function () { return e[k]; }
				});
			}
		});
	}
	n["default"] = e;
	return Object.freeze(n);
}

var path__default = /*#__PURE__*/_interopDefaultLegacy(path);
var fs__default = /*#__PURE__*/_interopDefaultLegacy(fs);
var yaml__namespace = /*#__PURE__*/_interopNamespace(yaml);
var https__default = /*#__PURE__*/_interopDefaultLegacy(https);

function defaults(target, source) {
  Object.keys(source).forEach((key) => {
    const sourceValue = source[key];
    if (sourceValue !== void 0) {
      target[key] = sourceValue;
    }
  });
  return target;
}
class K8sApi {
  constructor(clusterUrl, token) {
    this.token = token;
    this.clusterUrl = new url.URL(clusterUrl);
  }
  async request(path, options, body) {
    const defaultOptions = {
      hostname: this.clusterUrl.hostname,
      port: this.clusterUrl.port || "6443",
      path,
      method: "GET",
      headers: {},
      rejectUnauthorized: false
    };
    const finalOptions = defaults(options || {}, defaultOptions);
    defaults(finalOptions.headers, {
      Authorization: `Bearer ${this.token}`
    });
    return new Promise((fulfill, reject) => {
      const req = https__default["default"].request(finalOptions, (res) => {
        let data = "";
        res.on("data", (chunk) => data += chunk.toString());
        res.on(
          "end",
          () => fulfill({ statusCode: res.statusCode, data: JSON.parse(data) })
        );
      });
      req.on("error", reject);
      if (body) {
        req.write(body);
      }
      req.end();
    });
  }
  async get(path, options) {
    return this.request(path, defaults(options || {}, { method: "GET" }));
  }
  async post(path, object, options) {
    const body = JSON.stringify(object);
    const postOptions = defaults(options || {}, {
      method: "POST",
      headers: {
        "Content-Type": "application/json",
        "Content-Length": Buffer.byteLength(body)
      }
    });
    return this.request(path, postOptions, body);
  }
  async put(path, object, options) {
    const body = JSON.stringify(object);
    const putOptions = defaults(options || {}, {
      method: "PUT",
      headers: {
        "Content-Type": "application/json",
        "Content-Length": Buffer.byteLength(body)
      }
    });
    return this.request(path, putOptions, body);
  }
  async del(path, options) {
    return this.request(path, { ...options, method: "DELETE" });
  }
  getPluralPath(object) {
    return `/api${object.apiVersion.indexOf("/") > 0 ? "s" : ""}/${object.apiVersion}/namespaces/${object.metadata.namespace}/${object.kind.toLowerCase()}s`;
  }
  async applyObject(object) {
    let path = this.getPluralPath(object);
    return this.post(path, object).then((res) => {
      if (res.statusCode != 409) {
        return { message: `Object created successfully at ${path}`, data: res };
      }
      path += `/${object.metadata.name}`;
      return this.get(path).then(({ statusCode, data: getObject }) => {
        if (statusCode != 200)
          throw new Error(
            `k8s-apply cannot get object at ${path}: ${statusCode}
${JSON.stringify(
              getObject,
              null,
              4
            )}`
          );
        object.metadata.resourceVersion = getObject.metadata.resourceVersion;
        return this.put(path, object).then((updateRes) => {
          return {
            message: `Object updated successfully at ${path}`,
            data: updateRes
          };
        });
      });
    });
  }
  async delObject(object) {
    const path = this.getPluralPath(object) + `/${object.metadata.name}`;
    return this.del(path);
  }
}

const deployKubernetesAction = () => {
  return pluginScaffolderBackend.createTemplateAction({
    id: "deploy:kubernetes",
    description: "Deploys Kubernetes manifests.",
    schema: {
      input: zod.z.object({
        sourcePath: zod.z.string().optional(),
        targetPath: zod.z.string().optional(),
        authToken: zod.z.string().describe("Authentication token to access the Kubernetes cluster"),
        clusterUrl: zod.z.string().describe("URL of the Kubernetes cluster")
      })
    },
    async handler(ctx) {
      const { sourcePath, authToken, clusterUrl } = ctx.input;
      try {
        const k8sApi = new K8sApi(clusterUrl, authToken);
        const fileRoot = sourcePath ? backendCommon.resolveSafeChildPath(ctx.workspacePath, sourcePath) : ctx.workspacePath;
        const directoryContents = await pluginScaffolderNode.serializeDirectoryContents(fileRoot);
        const yamlFiles = directoryContents.filter(
          (file) => file.path.endsWith(".yaml") || file.path.endsWith(".yml")
        ).map((file) => path__default["default"].resolve(fileRoot, file.path));
        for (const filePath of yamlFiles) {
          try {
            const manifestYaml = await fs__default["default"].promises.readFile(filePath, {
              encoding: "utf-8"
            });
            yaml__namespace.loadAll(manifestYaml, (doc) => {
              try {
                const response = k8sApi.applyObject(doc);
                ctx.logger.info(
                  `Deployment response for ${filePath}: ${JSON.stringify(
                    response
                  )}`
                );
              } catch (error) {
                ctx.logger.error(
                  `Error applying Kubernetes object from ${filePath}`,
                  error
                );
              }
            });
          } catch (error) {
            ctx.logger.error(`Error processing file: ${filePath}`, error);
          }
        }
      } catch (error) {
        ctx.logger.error("An error occurred during deployment:", error);
        throw error;
      }
    }
  });
};

exports.deployKubernetesAction = deployKubernetesAction;
//# sourceMappingURL=index.cjs.js.map
